//
//  PingPongDelayNode.hpp
//  SwitchboardAudioEffects
//
//  Created by Balazs Kiss on 2022. 10. 13..
//

#pragma once

#include <switchboard_core/SingleBusAudioProcessorNode.hpp>

namespace switchboard::extensions::audioeffects {

// Enum for indexed accessing the API parameter objects in the `parameters` vector
// This is a temporary solution until we figure out the nicest way to passing API parameters around
enum PingPongDelayNodeParameters { Enabled = 0, DryMix = 1, WetMix = 2, DelayMs = 3, FeedbackLevel = 4 };

class PingPongDelayNode : public SingleBusAudioProcessorNode {
public:
    SB_WASM_EXPORT(PingPongDelayNode)

    SB_WASM PingPongDelayNode();
    SB_WASM virtual ~PingPongDelayNode();

    SB_WASM bool getIsEnabled() const;
    SB_WASM void setIsEnabled(bool newValue);

    SB_WASM float getDryMix() const;
    SB_WASM void setDryMix(float newValue);

    SB_WASM float getWetMix() const;
    SB_WASM void setWetMix(float newValue);

    SB_WASM uint getDelayMs() const;
    SB_WASM void setDelayMs(uint newValue);

    SB_WASM float getFeedbackLevel() const;
    SB_WASM void setFeedbackLevel(float newValue);

    // MARK: Overridden methods

    SB_WASM bool setBusFormat(AudioBusFormat& inputBusFormat, AudioBusFormat& outputBusFormat) override;
    SB_WASM bool process(AudioBus& inBus, AudioBus& outBus) override;

private:
    class Internals;
    std::unique_ptr<Internals> internals;

    void createParameters();

    template <class T, typename V>
    V getParameterValue(PingPongDelayNodeParameters type) const;

    template <class T, typename V>
    void setParameterValue(PingPongDelayNodeParameters type, V value);
};

}
